#include <stdio.h>
#include <limits.h>
#include "limits.h"
#include "linksinfo.h"
#include "commands.h"
#include "subscribe.h"
#include "unsubscribe.h"
#include "readconf.h"
#include "forward.h"
#include "sendmail.h"
#include "answer.h"

DoCommands(link_info *link)
{

 char request[CHAR_MAX];
 char c;

 log(4,"DoCommands.");
 
 while ( GetCmd(&c,request) )
   switch (c)
    {
    case 0: SendHelpMail(link);
     break;
    case 1: SendQueryMail(link);
     break;
    case 2: SendUnlinkedMail(link);
     break;
    case 3: SendListMail(link);
     break;
    case 4: SendAvail(link);
     break;
    case 5:
     if ( ! Subscribe(request,link) )
      {
       log(2,"%s not found in %s, try request uplink",request,conf.AreaFile);
       if ( ! Forward(request,link) )
        log(2,"%s not found in uplinks areafiles",request);
      }
     break;
    case 6: UnSubscribe(request,link);
     break;
    }
};



char AddCmd(char cmnum,char *param)
{

 cmds * tptr;

 tptr = ( cmds * ) malloc ( sizeof(cmds) );

 if ( tptr == NULL )
  return (1);

 tptr->cmdnum=cmnum;
 snprintf(tptr->cmdarg,"%s ",param,MAXLENECHO+1);
 tptr->nextcmd=cmdlist;

 cmdlist=tptr;

 log( 5, "AddCmd: Added cmd %d, param '%s' cmdlist %d ",cmnum, param ,cmdlist);
 return(0);
 
};

char GetCmd(char *cmnum,char *param)
{

 cmds * tptr;

 log(5,"GetCmd: cmdlist %d",cmdlist);
 
 if ( cmdlist == NULL )
  return (0);

 tptr=cmdlist;
 
 *cmnum=tptr->cmdnum;
 snprintf(param,"%s",tptr->cmdarg,MAXLENECHO+1);

 free(cmdlist);

 cmdlist=tptr->nextcmd;
 log( 5, "GetCmd: Got cmd %d, param %s ",cmnum, param );
 return(1);
 
};


SendHelpMail(link_info *links)
{
 FILE *af;
 char echo[MAXSTR+1];

 //   .
 log( 2, "Processing %HELP operation for %s",links->ftn_addr);

 OpenMail(links->name,links,"Help request");
 
 if( ( af = fopen( conf.HelpFile, "r" ) ) == NULL )
  {
   log( 0, "HelpFile \"%s\" cannot be opened!", conf.HelpFile );
   AddMail(links,NOHELPFILE);
   CloseMail(links);
   return( 1 );
  }

 while( fgets( echo, MAXSTR, af ) != NULL )
  {
   strtok (echo,"\n") ;
   AddMail(links,echo);
  }

 fclose(af);
 CloseMail(links);
 
 return( 0 );
}


/*				 %UNLINKED (  ! )	*/
SendUnlinkedMail(link_info *links)
{
 
 FILE *af;
 int n=0;

 char echo[MAXSTR+1];
 char echoname[MAXLENECHO+1];
 char tmpstr[CHAR_MAX];
 
 //   .
 log( 2, "Processing %unlinked operation for %s",links->ftn_addr);

 if( ( af = fopen( conf.AreaFile, "r" ) ) == NULL )
  {
   log( 0, "AreaFile \"%s\" cannot be opened!", conf.AreaFile );
   exit( 1 );
  }

 OpenMail(links->name,links,UNLINKEDREQUEST);

 while( fgets( echo, MAXSTR, af ) != NULL )
  {
   strtok (echo,"\n") ;

   if ( CheckEcho(echo,"*",tmpstr,echoname,links) == 0)
    {
     AddMail(links,echoname);
     n++;
    }
  }
 
 sprintf(echo,"%d areas currently %s for %s",n,NOTLINKED,links->ftn_addr);
 AddMail(links,echo);

 log( 3, "unlinked: %d areas currently %s for %s",n,NOTLINKED,links->ftn_addr);

 CloseMail(links);
 
 fclose(af);
 return( 0 );
}


/*				 %QUERY (  )	*/
SendQueryMail(link_info *links)
{
 
 FILE *af;
 int n=0;

 char echo[MAXSTR+1];
 char echoname[MAXLENECHO+1];
 char tmpstr[CHAR_MAX];
 
 //   .
 log( 2, "Processing %query operation for %s",links->ftn_addr);

 if( ( af = fopen( conf.AreaFile, "r" ) ) == NULL )
  {
   log( 0, "AreaFile \"%s\" cannot be opened!", conf.AreaFile );
   exit( 1 );
  }

 OpenMail(links->name,links,QUERYREQUEST);

 while( fgets( echo, MAXSTR, af ) != NULL )
  {
   strtok (echo,"\n") ;
      
   if ( CheckEcho(echo,"*",tmpstr,echoname,links) == 2)
    {
     sprintf(tmpstr,"* %s",echoname);
     AddMail(links,tmpstr);
     n++;
    }
  }
 
 sprintf(echo,"%d areas currently linked for %s",n,links->ftn_addr);
 AddMail(links,echo);

 log( 3, "query: %d areas currently linked for %s",n,links->ftn_addr);

 CloseMail(links);

 fclose(af);
 return( 0 );
}


/*		 %LIST      */
SendListMail(link_info *links)
{
 
 FILE *af;
 int n=0,nn=0;

 char echo[MAXSTR+1];
 char echoname[MAXLENECHO+1];
 char tmpstr[CHAR_MAX];
 
 //   .
 log( 2, "Processing %list operation for %s",links->ftn_addr);

 if( ( af = fopen( conf.AreaFile, "r" ) ) == NULL )
  {
   log( 0, "AreaFile \"%s\" cannot be opened!", conf.AreaFile );
   exit( 1 );
  }

 OpenMail(links->name,links,LISTREQUEST);
 while( fgets( echo, MAXSTR, af ) != NULL )
  {
   strtok (echo,"\n") ;

   switch ( CheckEcho(echo,"*",tmpstr,echoname,links) )
    {
    case 0:
     AddMail(links,echoname);
     nn++;
     break;
    case 2:
     sprintf(tmpstr,"* %s",echoname);
     AddMail(links,tmpstr);
     n++;
     nn++;
    }
  }
 
 sprintf(echo,"%d areas currently linked for %s",n,links->ftn_addr);
 AddMail(links,echo);

 log( 3, "list: %d areas avaible, %d linked for %s",nn,n,links->ftn_addr);

 CloseMail(links);

 fclose(af);
 return( 0 );
}

