#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <limits.h>
#include "linksinfo.h"
#include "readconf.h"
#include "ctype.h"

char FindLink(link_info *link)
{

 char zone[maxels+1];
 char net[maxels+1];
 char node[maxels+1];
 char pnt[maxels+1];
 char fadr[ftnsize+1];
 
 FILE *linksfile;

 if( ( linksfile = fopen( conf.LinksFile, "r" ) ) == NULL )
  {
   log(0, "Error! Cannot open links file %s",conf.LinksFile );
   return( 0 );
  }

 log(4,"FindLink: looking for %s",link->domain_addr);

 Domain2Ftn (link->domain_addr,fadr);

 log(4,"FindLink: ftn = %s",fadr);
 
 while ( GetLinksInfo(link,linksfile) ) //   EOF
  {
   log(5,"FindLink: got %s",link->ftn_addr);
   
   if ( ! strcmp(fadr,link->ftn_addr) )
    {
     fclose (linksfile);
     return (1);
    }
  }
   
 fclose (linksfile);
 return (0);
 
}
    

  
char GetLinksInfo(link_info *link,FILE *fl)
{
 
 int c;
 char buf[CHAR_MAX];
 char *comm;
 char *param;

 if ( feof (fl) ) return (0);

 strcpy(link->ftn_addr," ");
 strcpy(link->zone," ");
 strcpy(link->net," ");
 strcpy(link->node," ");
 strcpy(link->point,"0");
 
 strcpy(link->domain,"fidonet.org");
 strcpy(link->upasswd," ");
 strcpy(link->apasswd," ");

 strcpy(link->level,"0");
 strcpy(link->uareafix,"areafix");
 strcpy(link->forwardfile," ");


 while( fgets( buf, CHAR_MAX, fl ) != NULL )
  {

   strtok(buf,"\n");
   
   if( buf[0] != '#' )
    {
     comm=strtok(buf," \t");
     param=strtok(NULL," \t"); 
     
     c = FoundWord( comm );

     //printf("comm %d, param %s\n",c,param);

     switch( c )
      {
      case 0: strncpy( link->ftn_addr, param, ftnsize);
       //       printf("ftn: %s\n",link->ftn_addr);
       word2addr(link->ftn_addr,link->zone,link->net,link->node,link->point);
       break;
      case 1: strncpy( link->upasswd, param, MAXPASSLEN );
       break;
      case 2: strncpy( link->uareafix, param, CHAR_MAX );
       break;
      case 3: strncpy( link->apasswd, param, MAXPASSLEN );
       break;
      case 4:
       if ( isdigit(param[0]) )
        {
         snprintf(link->level,"%s",param,3);
        }
       else
        {
         log(1,"link %s has incorrect level: %c,assumed 0",link->ftn_addr,param[0]);
         sprintf(link->level,"0");
        }
       break;
      case 5: strncpy( link->forwardfile, param,PATH_MAX );
       break;
      case 6: return (1) ; //endlink
       break;
      default:
      }
    }
  }
}



FoundWord(char *command)
{
 int i=0;
 while( i++ != MAXLNKW )
  {
   strlwr( command );
   if( strcmp( command,LINKWORDS[i-1] ) == 0 )
    return( i-1 );
  }
 return( -1 );
}


Domain2Ftn ( char *adr,char *ftn)
{
 // 123:4506/789.123 
 // 123:4506/789 
 // p123.f789.n456.z123

 char zone[maxels+1];
 char net[maxels+1];
 char node[maxels+1];
 char pnt[maxels+1];

 char domain[CHAR_MAX];

 char *word;
 strcpy(zone,"zzzz");
 strcpy(net,"nnnn");
 strcpy(node,"ffff");
 strcpy(pnt,"0");

 strcpy(domain,adr);

 word=strtok(domain,".");
 while (word != NULL )
  {
   switch ( word[0] )
    {
    case 'p':
     word++;
     if ( pnt[0] == '0' )
      snprintf(pnt,"%s",word,maxels);
     break;
    case 'f':
     word++;
     if ( node[0] == 'f' )
      snprintf(node,"%s",word,maxels);
     break;
    case 'n':
     word++;
     if ( net[0] == 'n' )
      snprintf(net,"%s",word,maxels);
     break;
    case 'z':
     word++;
     if ( zone[0] == 'z' )
      snprintf(zone,"%s",word,maxels);
     break;
    }

   word=strtok(NULL,".");
  }

 if ( pnt[0]=='0')
  sprintf(ftn,"%s:%s/%s",zone,net,node);
 else
  sprintf(ftn,"%s:%s/%s.%s",zone,net,node,pnt);	
 //printf("ftn=%s\n",ftn);
}	

 
Ftn2Domain(char *ftn_in,char *domain)
{
 // 123:4506/789.123 
 // 123:4506/789 
 // p123.f789.n456.z123
 // f789.n456.z123

  
 char zone[maxels+1];
 char net[maxels+1];
 char node[maxels+1];
 char pnt[maxels+1];

 log(4,"ftn2domain: ftn %s",ftn_in);
 
 word2addr(ftn_in,zone,net,node,pnt);
 if ( (pnt[0] == '0') || (pnt[0] == ' ') )
  {
   sprintf(domain,"f%s.n%s.z%s",node,net,zone);
   //     printf("f%s.n%s.z%s\n",node,net,zone);
  }
 else
  {
   sprintf(domain,"p%s.f%s.n%s.z%s",pnt,node,net,zone);
  }

 //printf("domain=%s\n",domain);
 log(4,"ftn2domain: domain %s",domain);
 
}



word2addr(char *wword,char *wzone,char *wnet,char *wnode,char *wpnt)
{
 char bbuf[maxels*2+2];

 int wlen,wi0=0,wi1=0;

 strcpy(wzone,"zzz");
 strcpy(wnet,"nnnn");
 strcpy(wnode,"ffff");
 strcpy(wpnt,"0");
 
 log(5,"word2addr: word = %s",wword);

 wlen=strlen(wword);
 
 // printf("wword=%s\n",wword);
 
 // printf("wlen=%d\n",wlen);

 if ( wword[0] == '.' )
  {
   wword++;
   snprintf(wpnt,"%s",wword,maxels);
   
   log(5,"word2addr: zone = %s",wzone);
   log(5,"word2addr: net = %s",wnet);
   log(5,"word2addr: node = %s",wnode);
   log(5,"word2addr: pnt = %s",wpnt);
   
   return;
  }

 
 while ( wi0 <= wlen )
  {
   //   printf("%c\n",wword[wi0]);
   
   if ( isdigit (wword[wi0]) )
    {
     bbuf[wi1]=wword[wi0];
     wi0++;wi1++;
    }
   else
    {
     bbuf[wi1]='\0';
     wi1=0;

     //printf("%c\n",wword[wi0]);
     switch(wword[wi0])
      {
      case ':':
       snprintf(wzone,"%s",bbuf,maxels);
       break;
       
      case '/':
       snprintf(wnet,"%s",bbuf,maxels);
       break;
       
      case '.':
       snprintf(wnode,"%s",bbuf,maxels);
       break;
       
      default:
      }

     wi0++;
    }
  }


 if ( wnode[0] == 'f' )
  snprintf(wnode,"%s",bbuf,maxels);
 else
  snprintf(wpnt,"%s",bbuf,maxels);
  
 log(5,"word2addr: zone = %s",wzone);
 log(5,"word2addr: net = %s",wnet);
 log(5,"word2addr: node = %s",wnode);
 log(5,"word2addr: pnt = %s",wpnt);
}
